@@ -36,11 +36,11 @@
  */
 
 #include "dealer.h"
-
 #include "dealerinfo.h"
 #include "messagebox.h"
 #include "renderer.h"
 #include "speeds.h"
+#include "patsolve/solverinterface.h"
 #include "version.h"
 #include "view.h"
 
@@ -54,7 +54,6 @@
 #include <KSharedConfig>
 
 #include <QCoreApplication>
-#include <QMutex>
 #include <QThread>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
@@ -62,8 +61,6 @@
 
 #include <cmath>
 
-#define DEBUG_HINTS 0
-
 
 namespace
 {
@@ -93,21 +90,21 @@ namespace
     {
         switch ( status )
         {
-        case Solver::SolutionExists:
+        case SolverInterface::SolutionExists:
             return i18n("Solver: This game is winnable.");
-        case Solver::NoSolutionExists:
+        case SolverInterface::NoSolutionExists:
             return everWinnable ? i18n("Solver: This game is no longer winnable.")
                                 : i18n("Solver: This game cannot be won.");
-        case Solver::UnableToDetermineSolvability:
+        case SolverInterface::UnableToDetermineSolvability:
             return i18n("Solver: Unable to determine if this game is winnable.");
-        case Solver::SearchAborted:
-        case Solver::MemoryLimitReached:
+        case SolverInterface::SearchAborted:
+        case SolverInterface::MemoryLimitReached:
         default:
             return QString();
         }
     }
 
-    int readIntAttribute( const QXmlStreamReader & xml, const QString & key, bool * ok = 0 )
+    int readIntAttribute( const QXmlStreamReader & xml, const QString & key, bool * ok = nullptr )
     {
         QStringRef value = xml.attributes().value( key );
         return QString::fromRawData( value.data(), value.length() ).toInt( ok );
@@ -172,23 +169,20 @@ class SolverThread : public QThread
     Q_OBJECT
 
 public:
-    SolverThread( Solver * solver )
+    SolverThread( SolverInterface * solver )
       : m_solver( solver )
     {
     }
 
     void run() Q_DECL_OVERRIDE
     {
-        Solver::ExitStatus result = m_solver->patsolve();
+        SolverInterface::ExitStatus result = m_solver->patsolve();
         emit finished( result );
     }
 
     void abort()
     {
-        {
-            QMutexLocker lock( &m_solver->endMutex );
-            m_solver->m_shouldEnd = true;
-        }
+        m_solver->stopExecution();
         wait();
     }
 
@@ -196,7 +190,7 @@ signals:
     void finished( int result );
 
 private:
-    Solver * m_solver;
+    SolverInterface * m_solver;
 };
 
 
@@ -539,9 +533,9 @@ bool DealerScene::loadFile( QIODevice * io )
 
 DealerScene::DealerScene( const DealerInfo * di )
   : m_di( di ),
-    m_solver( 0 ),
-    m_solverThread( 0 ),
-    m_peekedCard( 0 ),
+    m_solver( nullptr ),
+    m_solverThread( nullptr ),
+    m_peekedCard( nullptr ),
     m_dealNumber( 0 ),
     m_loadedMoveCount( 0 ),
     m_neededFutureMoves( 1 ),
@@ -567,7 +561,7 @@ DealerScene::DealerScene( const DealerInfo * di )
     m_dropQueued( false ),
     m_newCardsQueued( false ),
     m_takeStateQueued( false ),
-    m_currentState( 0 )
+    m_currentState( nullptr )
 {
     setItemIndexMethod(QGraphicsScene::NoIndex);
 
@@ -601,9 +595,9 @@ DealerScene::~DealerScene()
     if ( m_solverThread )
         m_solverThread->abort();
     delete m_solverThread;
-    m_solverThread = 0;
+    m_solverThread = nullptr;
     delete m_solver;
-    m_solver = 0;
+    m_solver = nullptr;
     qDeleteAll( m_undoStack );
     delete m_currentState;
     qDeleteAll( m_redoStack );
@@ -710,13 +704,9 @@ QList<MoveHint> DealerScene::getSolverHints()
         m_solverThread->abort();
 
     solver()->translate_layout();
-    bool debug = false;
-#if DEBUG_HINTS
-    debug = true;
-#endif
-    solver()->patsolve( 1, debug );
+    solver()->patsolve( 1 );
 
-    foreach ( const MOVE & m, solver()->firstMoves )
+    foreach ( const MOVE & m, solver()->firstMoves() )
     {
         MoveHint mh = solver()->translateMove( m );
 	hintList << mh;
@@ -794,7 +784,7 @@ MoveHint DealerScene::chooseHint()
         MOVE m = m_winningMoves.takeFirst();
         MoveHint mh = solver()->translateMove( m );
 
-#if DEBUG_HINTS
+#ifndef NDEBUG
         if ( m.totype == O_Type )
             fprintf( stderr, "move from %d out (at %d) Prio: %d\n", m.from,
                      m.turn_index, m.pri );
@@ -874,7 +864,7 @@ void DealerScene::resetInternals()
     qDeleteAll( m_undoStack );
     m_undoStack.clear();
     delete m_currentState;
-    m_currentState = 0;
+    m_currentState = nullptr;
     qDeleteAll( m_redoStack );
     m_redoStack.clear();
     m_lastKnownCardStates.clear();
@@ -1089,7 +1079,7 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent * e )
     {
         e->accept();
         updatePileLayout( m_peekedCard->pile(), DURATION_FANCYSHOW );
-        m_peekedCard = 0;
+        m_peekedCard = nullptr;
     }
     else
     {
@@ -1238,8 +1228,8 @@ void DealerScene::undoOrRedo( bool undo )
 
         emit solverStateChanged( solverStatusMessage( solvability, m_dealWasEverWinnable ) );
 
-        if ( m_solver && ( solvability == Solver::SearchAborted
-                           || solvability == Solver::MemoryLimitReached ) )
+        if ( m_solver && ( solvability == SolverInterface::SearchAborted
+                           || solvability == SolverInterface::MemoryLimitReached ) )
         {
             startSolver();
         }
@@ -1505,9 +1495,9 @@ void DealerScene::slotSolverEnded()
 
 void DealerScene::slotSolverFinished( int result )
 {
-    if ( result == Solver::SolutionExists )
+    if ( result == SolverInterface::SolutionExists )
     {
-        m_winningMoves = m_solver->winMoves;
+        m_winningMoves = m_solver->winMoves();
         m_dealWasEverWinnable = true;
     }
 
@@ -1515,11 +1505,11 @@ void DealerScene::slotSolverFinished( int result )
 
     if ( m_currentState )
     {
-        m_currentState->solvability = static_cast<Solver::ExitStatus>( result );
+        m_currentState->solvability = static_cast<SolverInterface::ExitStatus>( result );
         m_currentState->winningMoves = m_winningMoves;
     }
 
-    if ( result == Solver::SearchAborted )
+    if ( result == SolverInterface::SearchAborted )
         startSolver();
 }
 
@@ -1708,11 +1698,11 @@ bool DealerScene::newCards()
 }
 
 
-void DealerScene::setSolver( Solver *s) {
+void DealerScene::setSolver( SolverInterface *s) {
     delete m_solver;
     delete m_solverThread;
     m_solver = s;
-    m_solverThread = 0;
+    m_solverThread = nullptr;
 }
 
 bool DealerScene::isGameWon() const
@@ -1734,13 +1724,17 @@ void DealerScene::startSolver()
 
 bool DealerScene::isGameLost() const
 {
+    if (! m_winningMoves.isEmpty())
+    {
+        return false;
+    }
     if ( solver() )
     {
         if ( m_solverThread && m_solverThread->isRunning() )
             m_solverThread->abort();
 
         solver()->translate_layout();
-        return solver()->patsolve( neededFutureMoves() ) == Solver::NoSolutionExists;
+        return solver()->patsolve( neededFutureMoves() ) == SolverInterface::NoSolutionExists;
     }
     return false;
 }
@@ -1830,7 +1824,7 @@ QList<QAction*> DealerScene::configActions() const
 }
 
 
-Solver * DealerScene::solver() const
+SolverInterface * DealerScene::solver() const
 {
     return m_solver;
 }
@@ -1892,7 +1886,7 @@ QImage DealerScene::createDump() const
         {
             p.save();
             p.setTransform( item->deviceTransform( p.worldTransform() ), false );
-            item->paint( &p, 0 );
+            item->paint( &p, nullptr );
             p.restore();
         }
     }
@@ -1955,7 +1949,7 @@ bool DealerScene::allowedToStartNewGame()
            || m_dealWasJustSaved
            || m_toldAboutWonGame
            || m_toldAboutLostGame
-           || KMessageBox::warningContinueCancel(0,
+           || KMessageBox::warningContinueCancel(nullptr,
                      i18n("A new game has been requested, but there is already a game in progress.\n\n"
                           "A loss will be recorded in the statistics if the current game is abandoned."),
                      i18n("Abandon Current Game?"),
