@@ -95,13 +95,13 @@ namespace
 
 
 MainWindow::MainWindow()
-  : KXmlGuiWindow( 0 ),
-    m_view( 0 ),
-    m_dealer( 0 ),
-    m_selector( 0 ),
-    m_cardDeck( 0 ),
-    m_soundEngine( 0 ),
-    m_dealDialog( 0 )
+  : KXmlGuiWindow( nullptr ),
+    m_view( nullptr ),
+    m_dealer( nullptr ),
+    m_selector( nullptr ),
+    m_cardDeck( nullptr ),
+    m_soundEngine( nullptr ),
+    m_dealDialog( nullptr )
 {
     setObjectName( QStringLiteral( "MainWindow" ) );
     // KCrash::setEmergencySaveFunction(::saveGame);
@@ -215,7 +215,7 @@ void MainWindow::setupActions()
     // action, so we must create a new action and copy all the standard
     // properties over one by one.
     m_hintAction = new KToggleAction( actionCollection() );
-    a = KStandardGameAction::hint( 0, 0, 0 );
+    a = KStandardGameAction::hint( nullptr, nullptr, nullptr );
     m_hintAction->setText( a->text() );
     m_hintAction->setIcon( a->icon() );
     actionCollection()->setDefaultShortcut( m_hintAction, a->shortcut() );
@@ -304,27 +304,27 @@ void MainWindow::setupActions()
 
     // Keyboard navigation actions
     m_leftAction = actionCollection()->addAction( QStringLiteral( "focus_left" ));
-    m_leftAction->setText(QStringLiteral("Move Focus to Previous Pile"));
+    m_leftAction->setText(i18n("Move Focus to Previous Pile"));
     actionCollection()->setDefaultShortcut(m_leftAction, Qt::Key_Left );
 
     m_rightAction = actionCollection()->addAction( QStringLiteral( "focus_right" ));
-    m_rightAction->setText(QStringLiteral("Move Focus to Next Pile"));
+    m_rightAction->setText(i18n("Move Focus to Next Pile"));
     actionCollection()->setDefaultShortcut(m_rightAction, Qt::Key_Right );
 
     m_upAction = actionCollection()->addAction( QStringLiteral( "focus_up" ));
-    m_upAction->setText(QStringLiteral("Move Focus to Card Below"));
+    m_upAction->setText(i18n("Move Focus to Card Below"));
     actionCollection()->setDefaultShortcut(m_upAction, Qt::Key_Up );
 
     m_downAction = actionCollection()->addAction( QStringLiteral( "focus_down" ));
-    m_downAction->setText(QStringLiteral("Move Focus to Card Above"));
+    m_downAction->setText(i18n("Move Focus to Card Above"));
     actionCollection()->setDefaultShortcut(m_downAction, Qt::Key_Down );
 
     m_cancelAction = actionCollection()->addAction( QStringLiteral( "focus_cancel" ));
-    m_cancelAction->setText(QStringLiteral("Cancel Focus"));
+    m_cancelAction->setText(i18n("Cancel Focus"));
     actionCollection()->setDefaultShortcut(m_cancelAction, Qt::Key_Escape );
 
     m_pickUpSetDownAction = actionCollection()->addAction( QStringLiteral( "focus_activate" ));
-    m_pickUpSetDownAction->setText(QStringLiteral("Pick Up or Set Down Focus"));
+    m_pickUpSetDownAction->setText(i18n("Pick Up or Set Down Focus"));
     actionCollection()->setDefaultShortcut(m_pickUpSetDownAction, Qt::Key_Space );
 
     // showMenubar isn't a part of KStandardGameAction
@@ -506,8 +506,8 @@ void MainWindow::setGameType(int id)
     {
         m_dealer->recordGameStatistics();
         delete m_dealer;
-        m_view->setScene(0);
-        m_dealer = 0;
+        m_view->setScene(nullptr);
+        m_dealer = nullptr;
     }
 
     if ( !m_cardDeck )
@@ -553,8 +553,8 @@ void MainWindow::slotShowGameSelectionScreen()
         {
             m_dealer->recordGameStatistics();
             delete m_dealer;
-            m_view->setScene(0);
-            m_dealer = 0;
+            m_view->setScene(nullptr);
+            m_dealer = nullptr;
         }
 
         if (!m_selector)
@@ -684,7 +684,7 @@ void MainWindow::updateSoundEngine()
         }
         else if ( m_soundEngine )
         {
-            disconnect( m_dealer, 0, m_soundEngine, 0 );
+            disconnect( m_dealer, nullptr, m_soundEngine, nullptr );
         }
     }
 }
